import logging

from django.views import generic
from django.conf import settings

from django_dragonpay.forms import *
from django_dragonpay.models import DragonpayTransaction, DragonpayPayout

logger = logging.getLogger('dragonpay.views')


__all__ = ['DragonpayCallbackBaseHandler', 'DragonpayPayoutBaseHandler']


class DragonpayCallbackBaseHandler(generic.View):
    '''Base view to handle Dragonpay callback transactions that
    updates the transactions in database.

    Usage:
        Inherit the class based view and implement your
        own GET and POST handlers

        class MyDragonpayCallback(DragonpayCallback):
            allow_invalid_data = False  # raise Error if data is invalid

            def get(self, request, *args, **kwargs):
                # the processed form may be access via
                self.form

                # do your stuff here
                pass

            def post(self, request, *args, **kwargs)
                pass
    '''

    allow_invalid_data = True    # should we crash if the data is invalid?
    update_on_GET = False        # should we update txns on GET request?

    def dispatch(self, *args, **kwargs):
        # if DRAGONPAY_SAVE_DATA settings is True, update the
        # DragonpayTransaction row for this transaction
        self.form = DragonpayCallbackForm(
            self.request.POST or self.request.GET)

        if not self.form.is_valid():
            logger.error(
                'Invalid Dragonpay callback request: %s', self.form.errors)

            if not self.allow_invalid_data:
                raise Exception('Invalid Dragonpay request')

        if self.request.method == 'GET' and not self.update_on_GET:
            # skip transaction updates if its a GET request
            pass

        elif settings.DRAGONPAY_SAVE_DATA:
            try:
                txn = DragonpayTransaction.objects.get(
                    id=self.form.cleaned_data['txnid'])
            except DragonpayTransaction.DoesNotExist as e:
                if not self.allow_invalid_data:
                    raise e
            else:
                # update the status of the transaction
                txn.status = self.form.cleaned_data['status']

                # set the reference number if it is still null
                if not txn.refno:
                    txn.refno = self.form.cleaned_data['refno']

                txn.save(update_fields=['status', 'refno'])
                logger.debug(
                    'Transaction %s updated to %s', txn.id, txn.status)
        return super(
            DragonpayCallbackBaseHandler, self).dispatch(*args, **kwargs)


class DragonpayPayoutBaseHandler(generic.View):
    allow_invalid_data = True    # should we crash if the data is invalid?

    def dispatch(self, request, *args, **kwargs):
        self.form = DragonpayPayoutCallbackForm(request.GET)

        if not self.form.is_valid():
            logger.error(
                'Invalid Dragonpay callback request: %s', self.form.errors)
            if not self.allow_invalid_data:
                raise Exception('Invalid Dragonpay request')

        if settings.DRAGONPAY_SAVE_DATA:
            try:
                txn = DragonpayPayout.objects.get(
                    id=self.form.cleaned_data['merchanttxnid'])
            except Exception as e:
                if not self.allow_invalid_data:
                    raise e
            else:
                txn.status = self.form.cleaned_data['status']
                txn.refno = self.form.cleaned_data['refno']

                txn.save(update_fields=['status', 'refno'])

                logger.debug('Payout %s updated to %s', txn.id, txn.status)
        return super(
            DragonpayPayoutBaseHandler, self).dispatch(
                request, *args, **kwargs)



import logging
from django import forms
from django_dragonpay.utils import decrypt_data
from django_dragonpay import settings as dp_settings
from django_dragonpay.utils import get_dragonpay_digest

logger = logging.getLogger('dragonpay.forms')


__all__ = ['DragonpayCallbackForm', 'DragonpayPayoutCallbackForm']


class DragonpayCallbackForm(forms.Form):
    '''Dragonpay form for Name-Value Pair model where data is sent via
    HTTP GET callbacks.'''

    STATUS_CODES = [
        ('S', 'Success'), ('F', 'Failure'), ('P', 'Pending'),
        ('U', 'Unknown'), ('R', 'Refund'), ('K', 'Chargeback'),
        ('V', 'Void'), ('A', 'Authorized')]

    txnid = forms.CharField(max_length=128)
    refno = forms.CharField(max_length=32)
    status = forms.CharField(max_length=1)
    message = forms.CharField(max_length=128)
    digest = forms.CharField(max_length=40)
    param1 = forms.CharField(max_length=80, required=False)
    param2 = forms.CharField(max_length=80, required=False)

    def clean(self):
        '''Custom clean method to verify the message authenticity thru
        the digest.'''

        KEYS = ['txnid', 'refno', 'status', 'message']
        try:
            to_digest = ':'.join([self.cleaned_data[key] for key in KEYS])
        except KeyError as e:
            logger.error('%s not found in request', e)
            raise forms.ValidationError('%s not found in request' % e)

        digest = get_dragonpay_digest(to_digest)

        # Validate that the message sent is cryptographically valid
        if self.cleaned_data['digest'] != digest:
            logger.error(
                'Request hash [%s] doesnt match caclulated [%s]',
                self.cleaned_data['digest'], digest)

            raise forms.ValidationError("DragonPay digest doesn't match!")

        # Decrypt params if they are encrypted
        if dp_settings.DRAGONPAY_ENCRYPT_PARAMS:
            for key in ['param1', 'param2']:
                param = self.cleaned_data.get(key)

                if param:
                    self.cleaned_data[key] = decrypt_data(param)
                    logger.debug(
                        'Decrypting %s:%s', param, self.cleaned_data[key])

        return self.cleaned_data


class DragonpayPayoutCallbackForm(forms.Form):
    refno = forms.CharField(max_length=32)
    status = forms.CharField(max_length=1)
    message = forms.CharField(max_length=128)
    merchanttxnid = forms.CharField(max_length=32)
    digest = forms.CharField(max_length=40)

    def __init__(self, data):
        # convert all keys to lowercase
        data = {i[0].lower(): i[1] for i in data.items()}
        logger.debug(data)
        super(DragonpayPayoutCallbackForm, self).__init__(data)

    def clean(self):
        KEYS = ['merchanttxnid', 'refno', 'status', 'message']

        try:
            to_digest = ':'.join([self.cleaned_data[key] for key in KEYS])
        except KeyError as e:
            logger.error('%s not found in request', e)
            raise forms.ValidationError('%s not found in request' % e)

        digest = get_dragonpay_digest(to_digest)

        # Validate that the message sent is cryptographically valid
        if self.cleaned_data['digest'] != digest:
            logger.error(
                'Request hash [%s] doesnt match caclulated [%s]',
                self.cleaned_data['digest'], digest)

            raise forms.ValidationError("DragonPay digest doesn't match!")

        return self.cleaned_data



from collections import OrderedDict
from django.conf import settings

# Dragonpay status codes
# See Appendix 3 of Dragonpay API documentation
DRAGONPAY_STATUS_CODES = {
    'S': 'Success',
    'F': 'Failed',
    'P': 'Pending',
    'U': 'Unknown',
    'R': 'Refund',
    'K': 'Chargeback',
    'V': 'Voided',
    'A': 'Authorized',
    'G': 'In progress',
}

# Dragonpay error codes.
# See Appendix 2 of Dragonpay API documentation
DRAGONPAY_ERROR_CODES = {
    '000': 'Success',
    '101': 'Invalid payment gateway id',
    '102': 'Incorrect secret key',
    '103': 'Invalid reference number',
    '104': 'Unauthorized access',
    '105': 'Invalid token',
    '106': 'Currency not supported',
    '107': 'Transaction cancelled',
    '108': 'Insufficient funds',
    '109': 'Transaction limit exceeded',
    '110': 'Error in operation',
    '111': 'Invalid parameters',
    '201': 'Invalid Merchant Id',
    '202': 'Invalid Merchant Password'
}


DRAGONPAY_PAYOUT_ERROR_CODES = {
    '0': 'Successfully created payout request',
    '-1': 'Invalid credentials or apiKey',
    '-2': '(reserved)',
    '-3': '(reserved)',
    '-4': 'Unable to create payout transaction (internal error)',
    '-5': 'Invalid account no / details',
    '-6': 'Invalid pre-dated run date',
    '-7': 'Amount exceeds limit for payout channel',
    '-8': 'A payout has been previously requested for the same merchant txn id',
    '-9': 'Source IP Address not whitelisted.'
}


DRAGONPAY_PAYMENT_METHODS = {
    'otc_bank': OrderedDict([
        ('BOGX', 'Bogus Bank Over-the-Counter'),
        ('BDOA', 'Banco de Oro ATM'),
        ('BDRX', 'BDO Cash Deposit'),
        ('BPXB', 'BPI Bills Payment'),
        ('MBTX', 'Metrobank Cash/Check Payment'),
        ('CBCX', 'Chinabank ATM/Cash Payment'),
        ('EWBX', 'EastWest Online/Cash/Check Payment'),
        ('LBXB', 'Landbank Cash Payment'),
        ('PNBB', 'PNB e-Banking Bills Payment'),
        ('PNXB', 'PNB Cash Payment'),
        ('RCXB', 'RCBC Cash Payment'),
        ('RSXB', 'RCBC Savings Cash Payment'),
        ('SBCA', 'Security Bank ATM Bills Payment'),
        ('SBCB', 'Security Bank Cash Payment'),
        ('UBXB', 'Unionbank Cash Payment'),
        ('UCXB', 'UCPB ATM/Cash Payment'),
    ]),
    'online_bank': OrderedDict([
        ('BOG', 'Bogus Bank'),
        ('BDO', 'BDO Internet Banking'),
        ('BPI', 'BPI Express Online (Fund Transfer)'),
        ('BPIB', 'BPI Express Online (Bills Payment)'),
        ('MBTC', 'Metrobank Direct Online'),
        ('CBC', 'Chinabank Online'),
        ('LBPA', 'Landbank iAccess'),
        ('RCBC', 'RCBC Online Banking'),
        ('UBP', 'UnionBank eBanking'),
        ('UCPB', 'UCPB Connect'),
    ]),
    'others': OrderedDict([
        ('BAYD', 'Bayad Center'),
        ('LBC', 'LBC'),
        ('SMR', 'SM Dept/Supermarket/Savemore Counter'),
        ('CEBL', 'Cebuana Lhuillier Bills Payment'),
        ('MLH', 'M. Lhuillier'),
        ('RDS', 'Robinsons Dept Store'),
        ('ECPY', 'ECPay (Pawnshops, Payment Centers)'),
        ('RLNT', 'RuralNet Banks and Coops'),
    ])
}

if not settings.DRAGONPAY_TEST_MODE:
    # Remove Bogus Bank from the Available Payment Methods
    DRAGONPAY_PAYMENT_METHODS['online_bank'].pop('BOG')
    DRAGONPAY_PAYMENT_METHODS['otc_bank'].pop('BOGX')

# Account number for supported PAYOUT channels
ACCOUNT_NUMBER_LENGTHS = {
    'BDO': 10, 'BPI': 10, 'CBC': 10, 'EWB': 12, 'LBP': 10,
    'MBTC': 13, 'RCBC': 10, 'SBC': 13, 'UBP': 12, 'UCPB': 12
}

ACCOUNT_NUMBER_REGEX = {
    'BDO': r'^(00)?\d{10}$', 'BPI': r'^\d{10}$', 'CBC': r'^\d{10}$',
    'EWB': r'^\d{12}$', 'LBP': r'^\d{10}$', 'MBTC': r'^\d{13}$',
    'RCBC': r'^\d{10}$', 'SBC': r'^\d{13}$', 'UBP': r'^\d{12}$',
    'UCPB': r'^\d{12}$'
}


# Dragonpay Paymemnt method FILTERS
ONLINE_BANKING = 1       # Online banking
OTC_BANK = 2             # Over-the-Counter Banking and ATM
OTC_NON_BANK = 4         # Over-the-Counter non-Bank
# 8 (unused)
# 16 (reserved internally)
PAYPAL = 32              # PayPal
CREDIT_CARDS = 64        # Credit Cards
MOBILE = 128             # Mobile (Gcash)
INTERNATIONAL_OTC = 256  # International OTC



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2017-07-10 03:18
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_dragonpay', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='dragonpaytransaction',
            name='status',
            field=models.CharField(choices=[(b'S', b'Success'), (b'F', b'Failed'), (b'P', b'Pending'), (b'U', b'Unknown'), (b'R', b'Refund'), (b'K', b'Chargeback'), (b'V', b'Voided'), (b'A', b'Authorized'), (b'E', b'Expired')], default=b'P', max_length=1),
        ),
    ]
